@@ -7,7 +7,7 @@
 # published by the Free Software Foundation.
 
 import sys
-import re
+from ..util import re_compile
 
 modules = [
     "2ch",
@@ -105,7 +105,6 @@ modules = [
     "mangadex",
     "mangafox",
     "mangahere",
-    "mangakakalot",
     "manganelo",
     "mangapark",
     "mangaread",
@@ -130,6 +129,7 @@ modules = [
     "philomena",
     "photovogue",
     "picarto",
+    "pictoa",
     "piczel",
     "pillowfort",
     "pinterest",
@@ -234,7 +234,8 @@ def find(url):
 
 def add(cls):
     """Add 'cls' to the list of available extractors"""
-    cls.pattern = re.compile(cls.pattern)
+    if isinstance(cls.pattern, str):
+        cls.pattern = re_compile(cls.pattern)
     _cache.append(cls)
     return cls
 
@@ -242,9 +243,11 @@ def add(cls):
 def add_module(module):
     """Add all extractors in 'module' to the list of available extractors"""
     classes = _get_classes(module)
-    for cls in classes:
-        cls.pattern = re.compile(cls.pattern)
-    _cache.extend(classes)
+    if classes:
+        if isinstance(classes[0].pattern, str):
+            for cls in classes:
+                cls.pattern = re_compile(cls.pattern)
+        _cache.extend(classes)
     return classes
 
 
