@@ -12,7 +12,8 @@ from .common import Extractor, Message
 from .. import text, util, exception
 from ..cache import cache, memcache
 
-BASE_PATTERN = r"(?:https?://)?bsky\.app"
+BASE_PATTERN = (r"(?:https?://)?"
+                r"(?:(?:www\.)?(?:c|[fv]x)?bs[ky]y[ex]?\.app|main\.bsky\.dev)")
 USER_PATTERN = BASE_PATTERN + r"/profile/([^/?#]+)"
 
 
@@ -24,10 +25,6 @@ class BlueskyExtractor(Extractor):
     archive_fmt = "{filename}"
     root = "https://bsky.app"
 
-    def __init__(self, match):
-        Extractor.__init__(self, match)
-        self.user = match.group(1)
-
     def _init(self):
         meta = self.config("metadata") or ()
         if meta:
@@ -42,62 +39,108 @@ class BlueskyExtractor(Extractor):
         self._user = self._user_did = None
         self.instance = self.root.partition("://")[2]
         self.videos = self.config("videos", True)
+        self.quoted = self.config("quoted", False)
 
     def items(self):
         for post in self.posts():
             if "post" in post:
                 post = post["post"]
-
-            pid = post["uri"].rpartition("/")[2]
             if self._user_did and post["author"]["did"] != self._user_did:
-                self.log.debug("Skipping %s (repost)", pid)
+                self.log.debug("Skipping %s (repost)", self._pid(post))
+                continue
+            embed = post.get("embed")
+            try:
+                post.update(post.pop("record"))
+            except Exception:
+                self.log.debug("Skipping %s (no 'record')", self._pid(post))
                 continue
 
-            post.update(post["record"])
-            del post["record"]
+            while True:
+                self._prepare(post)
+                files = self._extract_files(post)
 
-            if self._metadata_facets:
-                if "facets" in post:
-                    post["hashtags"] = tags = []
-                    post["mentions"] = dids = []
-                    post["uris"] = uris = []
-                    for facet in post["facets"]:
-                        features = facet["features"][0]
-                        if "tag" in features:
-                            tags.append(features["tag"])
-                        elif "did" in features:
-                            dids.append(features["did"])
-                        elif "uri" in features:
-                            uris.append(features["uri"])
-                else:
-                    post["hashtags"] = post["mentions"] = post["uris"] = ()
+                yield Message.Directory, post
+                if files:
+                    did = post["author"]["did"]
+                    base = (
+                        "{}/xrpc/com.atproto.sync.getBlob?did={}&cid=".format(
+                            self.api.service_endpoint(did), did))
+                    for post["num"], file in enumerate(files, 1):
+                        post.update(file)
+                        yield Message.Url, base + file["filename"], post
 
-            if self._metadata_user:
-                post["user"] = self._user or post["author"]
+                if not self.quoted or not embed or "record" not in embed:
+                    break
 
-            files = self._extract_files(post)
-            post["instance"] = self.instance
-            post["post_id"] = pid
-            post["count"] = len(files)
-            post["date"] = text.parse_datetime(
-                post["createdAt"][:19], "%Y-%m-%dT%H:%M:%S")
-
-            yield Message.Directory, post
-
-            if not files:
-                continue
-
-            base = ("https://bsky.social/xrpc/com.atproto.sync.getBlob"
-                    "?did={}&cid=".format(post["author"]["did"]))
-            for post["num"], file in enumerate(files, 1):
-                post.update(file)
-                yield Message.Url, base + file["filename"], post
+                quote = embed["record"]
+                if "record" in quote:
+                    quote = quote["record"]
+                value = quote.pop("value", None)
+                if value is None:
+                    break
+                quote["quote_id"] = self._pid(post)
+                quote["quote_by"] = post["author"]
+                embed = quote.get("embed")
+                quote.update(value)
+                post = quote
 
     def posts(self):
         return ()
 
+    def _posts_records(self, actor, collection):
+        depth = self.config("depth", "0")
+
+        for record in self.api.list_records(actor, collection):
+            uri = None
+            try:
+                uri = record["value"]["subject"]["uri"]
+                if "/app.bsky.feed.post/" in uri:
+                    yield from self.api.get_post_thread_uri(uri, depth)
+            except exception.StopExtraction:
+                pass  # deleted post
+            except Exception as exc:
+                self.log.debug(record, exc_info=exc)
+                self.log.warning("Failed to extract %s (%s: %s)",
+                                 uri or "record", exc.__class__.__name__, exc)
+
+    def _pid(self, post):
+        return post["uri"].rpartition("/")[2]
+
+    @memcache(keyarg=1)
+    def _instance(self, handle):
+        return ".".join(handle.rsplit(".", 2)[-2:])
+
+    def _prepare(self, post):
+        author = post["author"]
+        author["instance"] = self._instance(author["handle"])
+
+        if self._metadata_facets:
+            if "facets" in post:
+                post["hashtags"] = tags = []
+                post["mentions"] = dids = []
+                post["uris"] = uris = []
+                for facet in post["facets"]:
+                    features = facet["features"][0]
+                    if "tag" in features:
+                        tags.append(features["tag"])
+                    elif "did" in features:
+                        dids.append(features["did"])
+                    elif "uri" in features:
+                        uris.append(features["uri"])
+            else:
+                post["hashtags"] = post["mentions"] = post["uris"] = ()
+
+        if self._metadata_user:
+            post["user"] = self._user or author
+
+        post["instance"] = self.instance
+        post["post_id"] = self._pid(post)
+        post["date"] = text.parse_datetime(
+            post["createdAt"][:19], "%Y-%m-%dT%H:%M:%S")
+
     def _extract_files(self, post):
         if "embed" not in post:
+            post["count"] = 0
             return ()
 
         files = []
@@ -111,6 +154,7 @@ class BlueskyExtractor(Extractor):
         if "video" in media and self.videos:
             files.append(self._extract_media(media, "video"))
 
+        post["count"] = len(files)
         return files
 
     def _extract_media(self, media, key):
@@ -175,15 +219,19 @@ class BlueskyUserExtractor(BlueskyExtractor):
         pass
 
     def items(self):
-        base = "{}/profile/{}/".format(self.root, self.user)
+        base = "{}/profile/{}/".format(self.root, self.groups[0])
+        default = ("posts" if self.config("quoted", False) or
+                   self.config("reposts", False) else "media")
         return self._dispatch_extractors((
+            (BlueskyInfoExtractor      , base + "info"),
             (BlueskyAvatarExtractor    , base + "avatar"),
             (BlueskyBackgroundExtractor, base + "banner"),
             (BlueskyPostsExtractor     , base + "posts"),
             (BlueskyRepliesExtractor   , base + "replies"),
             (BlueskyMediaExtractor     , base + "media"),
+            (BlueskyVideoExtractor     , base + "video"),
             (BlueskyLikesExtractor     , base + "likes"),
-        ), ("media",))
+        ), (default,))
 
 
 class BlueskyPostsExtractor(BlueskyExtractor):
@@ -192,7 +240,8 @@ class BlueskyPostsExtractor(BlueskyExtractor):
     example = "https://bsky.app/profile/HANDLE/posts"
 
     def posts(self):
-        return self.api.get_author_feed(self.user, "posts_and_author_threads")
+        return self.api.get_author_feed(
+            self.groups[0], "posts_and_author_threads")
 
 
 class BlueskyRepliesExtractor(BlueskyExtractor):
@@ -201,7 +250,8 @@ class BlueskyRepliesExtractor(BlueskyExtractor):
     example = "https://bsky.app/profile/HANDLE/replies"
 
     def posts(self):
-        return self.api.get_author_feed(self.user, "posts_with_replies")
+        return self.api.get_author_feed(
+            self.groups[0], "posts_with_replies")
 
 
 class BlueskyMediaExtractor(BlueskyExtractor):
@@ -210,7 +260,18 @@ class BlueskyMediaExtractor(BlueskyExtractor):
     example = "https://bsky.app/profile/HANDLE/media"
 
     def posts(self):
-        return self.api.get_author_feed(self.user, "posts_with_media")
+        return self.api.get_author_feed(
+            self.groups[0], "posts_with_media")
+
+
+class BlueskyVideoExtractor(BlueskyExtractor):
+    subcategory = "video"
+    pattern = USER_PATTERN + r"/video"
+    example = "https://bsky.app/profile/HANDLE/video"
+
+    def posts(self):
+        return self.api.get_author_feed(
+            self.groups[0], "posts_with_video")
 
 
 class BlueskyLikesExtractor(BlueskyExtractor):
@@ -219,7 +280,9 @@ class BlueskyLikesExtractor(BlueskyExtractor):
     example = "https://bsky.app/profile/HANDLE/likes"
 
     def posts(self):
-        return self.api.get_actor_likes(self.user)
+        if self.config("endpoint") == "getActorLikes":
+            return self.api.get_actor_likes(self.groups[0])
+        return self._posts_records(self.groups[0], "app.bsky.feed.like")
 
 
 class BlueskyFeedExtractor(BlueskyExtractor):
@@ -227,12 +290,9 @@ class BlueskyFeedExtractor(BlueskyExtractor):
     pattern = USER_PATTERN + r"/feed/([^/?#]+)"
     example = "https://bsky.app/profile/HANDLE/feed/NAME"
 
-    def __init__(self, match):
-        BlueskyExtractor.__init__(self, match)
-        self.feed = match.group(2)
-
     def posts(self):
-        return self.api.get_feed(self.user, self.feed)
+        actor, feed = self.groups
+        return self.api.get_feed(actor, feed)
 
 
 class BlueskyListExtractor(BlueskyExtractor):
@@ -240,12 +300,9 @@ class BlueskyListExtractor(BlueskyExtractor):
     pattern = USER_PATTERN + r"/lists/([^/?#]+)"
     example = "https://bsky.app/profile/HANDLE/lists/ID"
 
-    def __init__(self, match):
-        BlueskyExtractor.__init__(self, match)
-        self.list = match.group(2)
-
     def posts(self):
-        return self.api.get_list_feed(self.user, self.list)
+        actor, list_id = self.groups
+        return self.api.get_list_feed(actor, list_id)
 
 
 class BlueskyFollowingExtractor(BlueskyExtractor):
@@ -254,7 +311,7 @@ class BlueskyFollowingExtractor(BlueskyExtractor):
     example = "https://bsky.app/profile/HANDLE/follows"
 
     def items(self):
-        for user in self.api.get_follows(self.user):
+        for user in self.api.get_follows(self.groups[0]):
             url = "https://bsky.app/profile/" + user["did"]
             user["_extractor"] = BlueskyUserExtractor
             yield Message.Queue, url, user
@@ -265,12 +322,20 @@ class BlueskyPostExtractor(BlueskyExtractor):
     pattern = USER_PATTERN + r"/post/([^/?#]+)"
     example = "https://bsky.app/profile/HANDLE/post/ID"
 
-    def __init__(self, match):
-        BlueskyExtractor.__init__(self, match)
-        self.post_id = match.group(2)
-
     def posts(self):
-        return self.api.get_post_thread(self.user, self.post_id)
+        actor, post_id = self.groups
+        return self.api.get_post_thread(actor, post_id)
+
+
+class BlueskyInfoExtractor(BlueskyExtractor):
+    subcategory = "info"
+    pattern = USER_PATTERN + r"/info"
+    example = "https://bsky.app/profile/HANDLE/info"
+
+    def items(self):
+        self._metadata_user = True
+        self.api._did_from_actor(self.groups[0])
+        return iter(((Message.Directory, self._user),))
 
 
 class BlueskyAvatarExtractor(BlueskyExtractor):
@@ -280,7 +345,7 @@ class BlueskyAvatarExtractor(BlueskyExtractor):
     example = "https://bsky.app/profile/HANDLE/avatar"
 
     def posts(self):
-        return self._make_post(self.user, "avatar")
+        return self._make_post(self.groups[0], "avatar")
 
 
 class BlueskyBackgroundExtractor(BlueskyExtractor):
@@ -290,7 +355,7 @@ class BlueskyBackgroundExtractor(BlueskyExtractor):
     example = "https://bsky.app/profile/HANDLE/banner"
 
     def posts(self):
-        return self._make_post(self.user, "banner")
+        return self._make_post(self.groups[0], "banner")
 
 
 class BlueskySearchExtractor(BlueskyExtractor):
@@ -299,13 +364,24 @@ class BlueskySearchExtractor(BlueskyExtractor):
     example = "https://bsky.app/search?q=QUERY"
 
     def posts(self):
-        return self.api.search_posts(self.user)
+        query = text.unquote(self.groups[0].replace("+", " "))
+        return self.api.search_posts(query)
+
+
+class BlueskyHashtagExtractor(BlueskyExtractor):
+    subcategory = "hashtag"
+    pattern = BASE_PATTERN + r"/hashtag/([^/?#]+)(?:/(top|latest))?"
+    example = "https://bsky.app/hashtag/NAME"
+
+    def posts(self):
+        hashtag, order = self.groups
+        return self.api.search_posts("#"+hashtag, order)
 
 
 class BlueskyAPI():
     """Interface for the Bluesky API
 
-    https://www.docs.bsky.app/docs/category/http-reference
+    https://docs.bsky.app/docs/category/http-reference
     """
 
     def __init__(self, extractor):
@@ -326,7 +402,7 @@ class BlueskyAPI():
             "actor": self._did_from_actor(actor),
             "limit": "100",
         }
-        return self._pagination(endpoint, params)
+        return self._pagination(endpoint, params, check_empty=True)
 
     def get_author_feed(self, actor, filter="posts_and_author_threads"):
         endpoint = "app.bsky.feed.getAuthorFeed"
@@ -364,11 +440,16 @@ class BlueskyAPI():
         return self._pagination(endpoint, params)
 
     def get_post_thread(self, actor, post_id):
+        uri = "at://{}/app.bsky.feed.post/{}".format(
+            self._did_from_actor(actor), post_id)
+        depth = self.extractor.config("depth", "0")
+        return self.get_post_thread_uri(uri, depth)
+
+    def get_post_thread_uri(self, uri, depth="0"):
         endpoint = "app.bsky.feed.getPostThread"
         params = {
-            "uri": "at://{}/app.bsky.feed.post/{}".format(
-                self._did_from_actor(actor), post_id),
-            "depth"       : self.extractor.config("depth", "0"),
+            "uri"         : uri,
+            "depth"       : depth,
             "parentHeight": "0",
         }
 
@@ -391,17 +472,46 @@ class BlueskyAPI():
         params = {"actor": did}
         return self._call(endpoint, params)
 
+    def list_records(self, actor, collection):
+        endpoint = "com.atproto.repo.listRecords"
+        actor_did = self._did_from_actor(actor)
+        params = {
+            "repo"      : actor_did,
+            "collection": collection,
+            "limit"     : "100",
+            #  "reverse"   : "false",
+        }
+        return self._pagination(endpoint, params, "records",
+                                self.service_endpoint(actor_did))
+
     @memcache(keyarg=1)
     def resolve_handle(self, handle):
         endpoint = "com.atproto.identity.resolveHandle"
         params = {"handle": handle}
         return self._call(endpoint, params)["did"]
 
-    def search_posts(self, query):
+    @memcache(keyarg=1)
+    def service_endpoint(self, did):
+        if did.startswith('did:web:'):
+            url = "https://" + did[8:] + "/.well-known/did.json"
+        else:
+            url = "https://plc.directory/" + did
+
+        try:
+            data = self.extractor.request(url).json()
+            for service in data["service"]:
+                if service["type"] == "AtprotoPersonalDataServer":
+                    return service["serviceEndpoint"]
+        except Exception:
+            pass
+        return "https://bsky.social"
+
+    def search_posts(self, query, sort=None):
         endpoint = "app.bsky.feed.searchPosts"
         params = {
             "q"    : query,
             "limit": "100",
+            "sort" : sort,
         }
         return self._pagination(endpoint, params, "posts")
 
@@ -415,7 +525,8 @@ class BlueskyAPI():
         if user_did and not extr.config("reposts", False):
             extr._user_did = did
         if extr._metadata_user:
-            extr._user = self.get_profile(did)
+            extr._user = user = self.get_profile(did)
+            user["instance"] = extr._instance(user["handle"])
 
         return did
 
@@ -453,8 +564,10 @@ class BlueskyAPI():
         _refresh_token_cache.update(self.username, data["refreshJwt"])
         return "Bearer " + data["accessJwt"]
 
-    def _call(self, endpoint, params):
-        url = "{}/xrpc/{}".format(self.root, endpoint)
+    def _call(self, endpoint, params, root=None):
+        if root is None:
+            root = self.root
+        url = "{}/xrpc/{}".format(root, endpoint)
 
         while True:
             self.authenticate()
@@ -479,9 +592,13 @@ class BlueskyAPI():
             self.extractor.log.debug("Server response: %s", response.text)
             raise exception.StopExtraction(msg)
 
-    def _pagination(self, endpoint, params, key="feed"):
+    def _pagination(self, endpoint, params,
+                    key="feed", root=None, check_empty=False):
         while True:
-            data = self._call(endpoint, params)
+            data = self._call(endpoint, params, root)
+
+            if check_empty and not data[key]:
+                return
             yield from data[key]
 
             cursor = data.get("cursor")
