@@ -26,10 +26,19 @@ class FanboxExtractor(Extractor):
     directory_fmt = ("{category}", "{creatorId}")
     filename_fmt = "{id}_{num}.{extension}"
     archive_fmt = "{id}_{num}"
+    browser = "firefox"
     _warning = True
 
     def _init(self):
-        self.headers = {"Origin": self.root}
+        self.headers = {
+            "Accept" : "application/json, text/plain, */*",
+            "Origin" : "https://www.fanbox.cc",
+            "Referer": "https://www.fanbox.cc/",
+            "Cookie" : None,
+            "Sec-Fetch-Dest": "empty",
+            "Sec-Fetch-Mode": "cors",
+            "Sec-Fetch-Site": "same-site",
+        }
         self.embeds = self.config("embeds", True)
 
         includes = self.config("metadata")
@@ -40,8 +49,12 @@ class FanboxExtractor(Extractor):
                 includes = ("user", "plan")
             self._meta_user = ("user" in includes)
             self._meta_plan = ("plan" in includes)
+            self._meta_comments = ("comments" in includes)
         else:
-            self._meta_user = self._meta_plan = False
+            self._meta_user = self._meta_plan = self._meta_comments = False
+
+        if self.config("comments"):
+            self._meta_comments = True
 
         if self._warning:
             if not self.cookies_check(("FANBOXSESSID",)):
@@ -113,7 +126,22 @@ class FanboxExtractor(Extractor):
             post["user"] = self._get_user_data(post["creatorId"])
         if self._meta_plan:
             plans = self._get_plan_data(post["creatorId"])
-            post["plan"] = plans[post["feeRequired"]]
+            fee = post["feeRequired"]
+            try:
+                post["plan"] = plans[fee]
+            except KeyError:
+                fees = [f for f in plans if f >= fee]
+                if fees:
+                    plan = plans[min(fees)]
+                else:
+                    plan = plans[0].copy()
+                    plan["fee"] = fee
+                post["plan"] = plans[fee] = plan
+        if self._meta_comments:
+            if post["commentCount"]:
+                post["comments"] = list(self._get_comment_data(post_id))
+            else:
+                post["commentd"] = ()
 
         return content_body, post
 
@@ -150,6 +178,19 @@ class FanboxExtractor(Extractor):
 
         return plans
 
+    def _get_comment_data(self, post_id):
+        url = ("https://api.fanbox.cc/post.getComments"
+               "?limit=10&postId=" + post_id)
+
+        comments = []
+        while url:
+            url = text.ensure_http_scheme(url)
+            body = self.request(url, headers=self.headers).json()["body"]
+            data = body["commentList"]
+            comments.extend(data["items"])
+            url = data["nextUrl"]
+        return comments
+
     def _get_urls_from_post(self, content_body, post):
         num = 0
         cover_image = post.get("coverImageUrl")
@@ -262,8 +303,7 @@ class FanboxExtractor(Extractor):
             url = "https://www.pixiv.net/fanbox/"+content_id
             # resolve redirect
             try:
-                url = self.request(url, method="HEAD",
-                                   allow_redirects=False).headers["location"]
+                url = self.request_location(url)
             except Exception as exc:
                 url = None
                 self.log.warning("Unable to extract fanbox embed %s (%s: %s)",
@@ -299,8 +339,20 @@ class FanboxCreatorExtractor(FanboxExtractor):
         self.creator_id = match.group(1) or match.group(2)
 
     def posts(self):
-        url = "https://api.fanbox.cc/post.listCreator?creatorId={}&limit=10"
-        return self._pagination(url.format(self.creator_id))
+        url = "https://api.fanbox.cc/post.paginateCreator?creatorId="
+        return self._pagination_creator(url + self.creator_id)
+
+    def _pagination_creator(self, url):
+        urls = self.request(url, headers=self.headers).json()["body"]
+        for url in urls:
+            url = text.ensure_http_scheme(url)
+            body = self.request(url, headers=self.headers).json()["body"]
+            for item in body:
+                try:
+                    yield self._get_post_data(item["id"])
+                except Exception as exc:
+                    self.log.warning("Skipping post %s (%s: %s)",
+                                     item["id"], exc.__class__.__name__, exc)
 
 
 class FanboxPostExtractor(FanboxExtractor):
@@ -346,13 +398,7 @@ class FanboxRedirectExtractor(Extractor):
     pattern = r"(?:https?://)?(?:www\.)?pixiv\.net/fanbox/creator/(\d+)"
     example = "https://www.pixiv.net/fanbox/creator/12345"
 
-    def __init__(self, match):
-        Extractor.__init__(self, match)
-        self.user_id = match.group(1)
-
     def items(self):
-        url = "https://www.pixiv.net/fanbox/creator/" + self.user_id
-        data = {"_extractor": FanboxCreatorExtractor}
-        response = self.request(
-            url, method="HEAD", allow_redirects=False, notfound="user")
-        yield Message.Queue, response.headers["Location"], data
+        url = "https://www.pixiv.net/fanbox/creator/" + self.groups[0]
+        location = self.request_location(url, notfound="user")
+        yield Message.Queue, location, {"_extractor": FanboxCreatorExtractor}
