@@ -11,7 +11,7 @@
 from .common import Extractor, Message
 from .. import text, util
 
-BASE_PATTERN = r"(?:https?://)?(?:www\.|sfw\.)?furaffinity\.net"
+BASE_PATTERN = r"(?:https?://)?(?:www\.|sfw\.)?(?:f[ux]|f?xfu)raffinity\.net"
 
 
 class FuraffinityExtractor(Extractor):
@@ -20,8 +20,10 @@ class FuraffinityExtractor(Extractor):
     directory_fmt = ("{category}", "{user!l}")
     filename_fmt = "{id}{title:? //}.{extension}"
     archive_fmt = "{id}"
-    cookiedomain = ".furaffinity.net"
+    cookies_domain = ".furaffinity.net"
+    cookies_names = ("a", "b")
     root = "https://www.furaffinity.net"
+    request_interval = 1.0
     _warning = True
 
     def __init__(self, match):
@@ -29,6 +31,9 @@ class FuraffinityExtractor(Extractor):
         self.user = match.group(1)
         self.offset = 0
 
+    def _init(self):
+        self.external = self.config("external", False)
+
         if self.config("descriptions") == "html":
             self._process_description = str.strip
 
@@ -38,14 +43,12 @@ class FuraffinityExtractor(Extractor):
         else:
             self._new_layout = None
 
-    def items(self):
-
         if self._warning:
-            if not self._check_cookies(("a", "b")):
+            if not self.cookies_check(self.cookies_names):
                 self.log.warning("no 'a' and 'b' session cookies set")
             FuraffinityExtractor._warning = False
 
-        external = self.config("external", False)
+    def items(self):
         metadata = self.metadata()
         for post_id in util.advance(self.posts(), self.offset):
             post = self._parse_post(post_id)
@@ -55,7 +58,7 @@ class FuraffinityExtractor(Extractor):
                 yield Message.Directory, post
                 yield Message.Url, post["url"], post
 
-                if external:
+                if self.external:
                     for url in text.extract_iter(
                             post["_description"], 'href="http', '"'):
                         yield Message.Queue, "http" + url, post
@@ -76,14 +79,12 @@ class FuraffinityExtractor(Extractor):
 
         path = extr('href="//d', '"')
         if not path:
-            self.log.warning(
-                "Unable to download post %s (\"%s\")",
-                post_id, text.remove_html(
-                    extr('System Message', '</section>') or
-                    extr('System Message', '</table>')
-                )
-            )
-            return None
+            msg = text.remove_html(
+                extr('System Message', '</section>') or
+                extr('System Message', '</table>')
+            ).partition(" . Continue ")[0]
+            return self.log.warning(
+                "Unable to download post %s (\"%s\")", post_id, msg)
 
         pi = text.parse_int
         rh = text.remove_html
@@ -96,8 +97,10 @@ class FuraffinityExtractor(Extractor):
         if self._new_layout:
             data["tags"] = text.split_html(extr(
                 'class="tags-row">', '</section>'))
+            data["scraps"] = (extr(' submissions">', "<") == "Scraps")
             data["title"] = text.unescape(extr("<h2><p>", "</p></h2>"))
-            data["artist"] = extr("<strong>", "<")
+            data["artist_url"] = extr('title="', '"').strip()
+            data["artist"] = extr(">", "<")
             data["_description"] = extr(
                 'class="submission-description user-submitted-links">',
                 '                                    </div>')
@@ -111,9 +114,18 @@ class FuraffinityExtractor(Extractor):
             data["gender"] = rh(extr('>Gender</strong>', '</div>'))
             data["width"] = pi(extr("<span>", "x"))
             data["height"] = pi(extr("", "p"))
+            data["folders"] = folders = []
+            for folder in extr(
+                    "<h3>Listed in Folders</h3>", "</section>").split("</a>"):
+                folder = rh(folder)
+                if folder:
+                    folders.append(folder)
         else:
             # old site layout
+            data["scraps"] = (
+                "/scraps/" in extr('class="minigallery-title', "</a>"))
             data["title"] = text.unescape(extr("<h2>", "</h2>"))
+            data["artist_url"] = extr('title="', '"').strip()
             data["artist"] = extr(">", "<")
             data["fa_category"] = extr("<b>Category:</b>", "<").strip()
             data["theme"] = extr("<b>Theme:</b>", "<").strip()
@@ -130,11 +142,13 @@ class FuraffinityExtractor(Extractor):
             data["_description"] = extr(
                 '<td valign="top" align="left" width="70%" class="alt1" '
                 'style="padding:8px">', '                               </td>')
+            data["folders"] = ()  # folders not present in old layout
 
-        data["artist_url"] = data["artist"].replace("_", "").lower()
         data["user"] = self.user or data["artist_url"]
         data["date"] = text.parse_timestamp(data["filename"].partition(".")[0])
         data["description"] = self._process_description(data["_description"])
+        data["thumbnail"] = "https://t.furaffinity.net/{}@600-{}.jpg".format(
+            post_id, path.rsplit("/", 2)[1])
 
         return data
 
@@ -142,12 +156,13 @@ class FuraffinityExtractor(Extractor):
     def _process_description(description):
         return text.unescape(text.remove_html(description, "", ""))
 
-    def _pagination(self, path):
+    def _pagination(self, path, folder=None):
         num = 1
+        folder = "" if folder is None else "/folder/{}/a".format(folder)
 
         while True:
-            url = "{}/{}/{}/{}/".format(
-                self.root, path, self.user, num)
+            url = "{}/{}/{}{}/{}/".format(
+                self.root, path, self.user, folder, num)
             page = self.request(url).text
             post_id = None
 
@@ -170,6 +185,11 @@ class FuraffinityExtractor(Extractor):
                     break
                 self._favorite_id = text.parse_int(extr('data-fav-id="', '"'))
                 yield post_id
+
+            pos = page.find('type="submit">Next</button>')
+            if pos >= 0:
+                path = text.rextract(page, '<form action="', '"', pos)[0]
+                continue
             path = text.extr(page, 'right" href="', '"')
 
     def _pagination_search(self, query):
@@ -216,28 +236,37 @@ class FuraffinityExtractor(Extractor):
 class FuraffinityGalleryExtractor(FuraffinityExtractor):
     """Extractor for a furaffinity user's gallery"""
     subcategory = "gallery"
-    pattern = BASE_PATTERN + r"/gallery/([^/?#]+)"
-    test = ("https://www.furaffinity.net/gallery/mirlinthloth/", {
-        "pattern": r"https://d\d?\.f(uraffinity|acdn)\.net"
-                   r"/art/mirlinthloth/\d+/\d+.\w+\.\w+",
-        "range": "45-50",
-        "count": 6,
-    })
+    pattern = BASE_PATTERN + r"/gallery/([^/?#]+)(?:$|/(?!folder/))"
+    example = "https://www.furaffinity.net/gallery/USER/"
 
     def posts(self):
         return self._pagination("gallery")
 
 
+class FuraffinityFolderExtractor(FuraffinityExtractor):
+    """Extractor for a FurAffinity folder"""
+    subcategory = "folder"
+    directory_fmt = ("{category}", "{user!l}",
+                     "Folders", "{folder_id}{folder_name:? //}")
+    pattern = BASE_PATTERN + r"/gallery/([^/?#]+)/folder/(\d+)(?:/([^/?#]+))?"
+    example = "https://www.furaffinity.net/gallery/USER/folder/12345/FOLDER"
+
+    def metadata(self):
+        return {
+            "folder_id"  : self.groups[1],
+            "folder_name": self.groups[2] or "",
+        }
+
+    def posts(self):
+        return self._pagination("gallery", self.groups[1])
+
+
 class FuraffinityScrapsExtractor(FuraffinityExtractor):
     """Extractor for a furaffinity user's scraps"""
     subcategory = "scraps"
     directory_fmt = ("{category}", "{user!l}", "Scraps")
     pattern = BASE_PATTERN + r"/scraps/([^/?#]+)"
-    test = ("https://www.furaffinity.net/scraps/mirlinthloth/", {
-        "pattern": r"https://d\d?\.f(uraffinity|acdn)\.net"
-                   r"/art/[^/]+(/stories)?/\d+/\d+.\w+.",
-        "count": ">= 3",
-    })
+    example = "https://www.furaffinity.net/scraps/USER/"
 
     def posts(self):
         return self._pagination("scraps")
@@ -248,13 +277,7 @@ class FuraffinityFavoriteExtractor(FuraffinityExtractor):
     subcategory = "favorite"
     directory_fmt = ("{category}", "{user!l}", "Favorites")
     pattern = BASE_PATTERN + r"/favorites/([^/?#]+)"
-    test = ("https://www.furaffinity.net/favorites/mirlinthloth/", {
-        "pattern": r"https://d\d?\.f(uraffinity|acdn)\.net"
-                   r"/art/[^/]+/\d+/\d+.\w+\.\w+",
-        "keyword": {"favorite_id": int},
-        "range": "45-50",
-        "count": 6,
-    })
+    example = "https://www.furaffinity.net/favorites/USER/"
 
     def posts(self):
         return self._pagination_favorites()
@@ -271,19 +294,7 @@ class FuraffinitySearchExtractor(FuraffinityExtractor):
     subcategory = "search"
     directory_fmt = ("{category}", "Search", "{search}")
     pattern = BASE_PATTERN + r"/search(?:/([^/?#]+))?/?[?&]([^#]+)"
-    test = (
-        ("https://www.furaffinity.net/search/?q=cute", {
-            "pattern": r"https://d\d?\.f(uraffinity|acdn)\.net"
-                       r"/art/[^/]+/\d+/\d+.\w+\.\w+",
-            "range": "45-50",
-            "count": 6,
-        }),
-        # first page of search results (#2402)
-        ("https://www.furaffinity.net/search/?q=leaf&range=1day", {
-            "range": "1-3",
-            "count": 3,
-        }),
-    )
+    example = "https://www.furaffinity.net/search/?q=QUERY"
 
     def __init__(self, match):
         FuraffinityExtractor.__init__(self, match)
@@ -302,65 +313,7 @@ class FuraffinityPostExtractor(FuraffinityExtractor):
     """Extractor for individual posts on furaffinity"""
     subcategory = "post"
     pattern = BASE_PATTERN + r"/(?:view|full)/(\d+)"
-    test = (
-        ("https://www.furaffinity.net/view/21835115/", {
-            "pattern": r"https://d\d*\.f(uraffinity|acdn)\.net/(download/)?art"
-                       r"/mirlinthloth/music/1488278723/1480267446.mirlinthlot"
-                       r"h_dj_fennmink_-_bude_s_4_ever\.mp3",
-            "keyword": {
-                "artist"     : "mirlinthloth",
-                "artist_url" : "mirlinthloth",
-                "date"       : "dt:2016-11-27 17:24:06",
-                "description": "A Song made playing the game Cosmic DJ.",
-                "extension"  : "mp3",
-                "filename"   : r"re:\d+\.\w+_dj_fennmink_-_bude_s_4_ever",
-                "id"         : 21835115,
-                "tags"       : list,
-                "title"      : "Bude's 4 Ever",
-                "url"        : r"re:https://d\d?\.f(uraffinity|acdn)\.net/art",
-                "user"       : "mirlinthloth",
-                "views"      : int,
-                "favorites"  : int,
-                "comments"   : int,
-                "rating"     : "General",
-                "fa_category": "Music",
-                "theme"      : "All",
-                "species"    : "Unspecified / Any",
-                "gender"     : "Any",
-                "width"      : 120,
-                "height"     : 120,
-            },
-        }),
-        # 'external' option (#1492)
-        ("https://www.furaffinity.net/view/42166511/", {
-            "options": (("external", True),),
-            "pattern": r"https://d\d*\.f(uraffinity|acdn)\.net/"
-                       r"|http://www\.postybirb\.com",
-            "count": 2,
-        }),
-        # no tags (#2277)
-        ("https://www.furaffinity.net/view/45331225/", {
-            "keyword": {
-                "artist": "Kota_Remminders",
-                "artist_url": "kotaremminders",
-                "date": "dt:2022-01-03 17:49:33",
-                "fa_category": "Adoptables",
-                "filename": "1641232173.kotaremminders_chidopts1",
-                "gender": "Any",
-                "height": 905,
-                "id": 45331225,
-                "rating": "General",
-                "species": "Unspecified / Any",
-                "tags": [],
-                "theme": "All",
-                "title": "REMINDER",
-                "width": 1280,
-            },
-        }),
-        ("https://furaffinity.net/view/21835115/"),
-        ("https://sfw.furaffinity.net/view/21835115/"),
-        ("https://www.furaffinity.net/full/21835115/"),
-    )
+    example = "https://www.furaffinity.net/view/12345/"
 
     def posts(self):
         post_id = self.user
@@ -371,18 +324,14 @@ class FuraffinityPostExtractor(FuraffinityExtractor):
 class FuraffinityUserExtractor(FuraffinityExtractor):
     """Extractor for furaffinity user profiles"""
     subcategory = "user"
-    cookiedomain = None
+    cookies_domain = None
     pattern = BASE_PATTERN + r"/user/([^/?#]+)"
-    test = (
-        ("https://www.furaffinity.net/user/mirlinthloth/", {
-            "pattern": r"/gallery/mirlinthloth/$",
-        }),
-        ("https://www.furaffinity.net/user/mirlinthloth/", {
-            "options": (("include", "all"),),
-            "pattern": r"/(gallery|scraps|favorites)/mirlinthloth/$",
-            "count": 3,
-        }),
-    )
+    example = "https://www.furaffinity.net/user/USER/"
+
+    def initialize(self):
+        pass
+
+    skip = Extractor.skip
 
     def items(self):
         base = "{}/{{}}/{}/".format(self.root, self.user)
@@ -397,11 +346,7 @@ class FuraffinityFollowingExtractor(FuraffinityExtractor):
     """Extractor for a furaffinity user's watched users"""
     subcategory = "following"
     pattern = BASE_PATTERN + "/watchlist/by/([^/?#]+)"
-    test = ("https://www.furaffinity.net/watchlist/by/mirlinthloth/", {
-        "pattern": FuraffinityUserExtractor.pattern,
-        "range": "176-225",
-        "count": 50,
-    })
+    example = "https://www.furaffinity.net/watchlist/by/USER/"
 
     def items(self):
         url = "{}/watchlist/by/{}/".format(self.root, self.user)
@@ -417,3 +362,29 @@ class FuraffinityFollowingExtractor(FuraffinityExtractor):
             if url.endswith(path):
                 return
             url = self.root + path
+
+
+class FuraffinitySubmissionsExtractor(FuraffinityExtractor):
+    """Extractor for new furaffinity submissions"""
+    subcategory = "submissions"
+    pattern = BASE_PATTERN + r"(/msg/submissions(?:/[^/?#]+)?)"
+    example = "https://www.furaffinity.net/msg/submissions"
+
+    def posts(self):
+        self.user = None
+        url = self.root + self.groups[0]
+        return self._pagination_submissions(url)
+
+    def _pagination_submissions(self, url):
+        while True:
+            page = self.request(url).text
+
+            for post_id in text.extract_iter(page, 'id="sid-', '"'):
+                yield post_id
+
+            path = (text.extr(page, '<a class="button standard more" href="', '"') or  # noqa 501
+                    text.extr(page, '<a class="more-half" href="', '"') or
+                    text.extr(page, '<a class="more" href="', '"'))
+            if not path:
+                return
+            url = self.root + text.unescape(path)
