@@ -29,6 +29,7 @@ class InstagramExtractor(Extractor):
     root = "https://www.instagram.com"
     cookies_domain = ".instagram.com"
     cookies_names = ("sessionid",)
+    useragent = util.USERAGENT_CHROME
     request_interval = (6.0, 12.0)
 
     def __init__(self, match):
@@ -164,13 +165,16 @@ class InstagramExtractor(Extractor):
         if "items" in post:  # story or highlight
             items = post["items"]
             reel_id = str(post["id"]).rpartition(":")[2]
+            expires = post.get("expiring_at")
             data = {
-                "expires": text.parse_timestamp(post.get("expiring_at")),
+                "expires": text.parse_timestamp(expires),
                 "post_id": reel_id,
                 "post_shortcode": shortcode_from_id(reel_id),
             }
             if "title" in post:
                 data["highlight_title"] = post["title"]
+            if expires and not post.get("seen"):
+                post["seen"] = expires - 86400
 
         else:  # regular image/video post
             data = {
@@ -582,7 +586,10 @@ class InstagramStoriesExtractor(InstagramExtractor):
         reel_id = self.highlight_id or self.api.user_id(self.user)
         reels = self.api.reels_media(reel_id)
 
-        if self.media_id and reels:
+        if not reels:
+            return ()
+
+        if self.media_id:
             reel = reels[0]
             for item in reel["items"]:
                 if item["pk"] == self.media_id:
@@ -591,6 +598,16 @@ class InstagramStoriesExtractor(InstagramExtractor):
             else:
                 raise exception.NotFoundError("story")
 
+        elif self.config("split"):
+            reel = reels[0]
+            reels = []
+            for item in reel["items"]:
+                item.pop("user", None)
+                copy = reel.copy()
+                copy.update(item)
+                copy["items"] = (item,)
+                reels.append(copy)
+
         return reels
 
 
@@ -605,6 +622,20 @@ class InstagramHighlightsExtractor(InstagramExtractor):
         return self.api.highlights_media(uid)
 
 
+class InstagramFollowersExtractor(InstagramExtractor):
+    """Extractor for an Instagram user's followers"""
+    subcategory = "followers"
+    pattern = USER_PATTERN + r"/followers"
+    example = "https://www.instagram.com/USER/followers/"
+
+    def items(self):
+        uid = self.api.user_id(self.item)
+        for user in self.api.user_followers(uid):
+            user["_extractor"] = InstagramUserExtractor
+            url = "{}/{}".format(self.root, user["username"])
+            yield Message.Queue, url, user
+
+
 class InstagramFollowingExtractor(InstagramExtractor):
     """Extractor for an Instagram user's followed users"""
     subcategory = "following"
@@ -692,11 +723,21 @@ class InstagramPostExtractor(InstagramExtractor):
     """Extractor for an Instagram post"""
     subcategory = "post"
     pattern = (r"(?:https?://)?(?:www\.)?instagram\.com"
-               r"/(?:[^/?#]+/)?(?:p|tv|reel)/([^/?#]+)")
+               r"/(?:share/()|[^/?#]+/)?(?:p|tv|reel)/([^/?#]+)")
     example = "https://www.instagram.com/p/abcdefg/"
 
     def posts(self):
-        return self.api.media(self.item)
+        share, shortcode = self.groups
+        if share is not None:
+            url = text.ensure_http_scheme(self.url)
+            headers = {
+                "Sec-Fetch-Dest": "empty",
+                "Sec-Fetch-Mode": "navigate",
+                "Sec-Fetch-Site": "same-origin",
+            }
+            location = self.request_location(url, headers=headers)
+            shortcode = location.split("/")[-2]
+        return self.api.media(shortcode)
 
 
 class InstagramRestAPI():
@@ -815,6 +856,11 @@ class InstagramRestAPI():
         params = {"count": 30}
         return self._pagination(endpoint, params)
 
+    def user_followers(self, user_id):
+        endpoint = "/v1/friendships/{}/followers/".format(user_id)
+        params = {"count": 12}
+        return self._pagination_following(endpoint, params)
+
     def user_following(self, user_id):
         endpoint = "/v1/friendships/{}/following/".format(user_id)
         params = {"count": 12}
@@ -907,9 +953,10 @@ class InstagramRestAPI():
             for item in data["items"]:
                 yield from item["media_items"]
 
-            if "next_max_id" not in data:
+            next_max_id = data.get("next_max_id")
+            if not next_max_id:
                 return extr._update_cursor(None)
-            params["max_id"] = extr._update_cursor(data["next_max_id"])
+            params["max_id"] = extr._update_cursor(next_max_id)
 
     def _pagination_following(self, endpoint, params):
         extr = self.extractor
@@ -920,10 +967,10 @@ class InstagramRestAPI():
 
             yield from data["users"]
 
-            if len(data["users"]) < params["count"]:
+            next_max_id = data.get("next_max_id")
+            if not next_max_id:
                 return extr._update_cursor(None)
-            params["max_id"] = extr._update_cursor(
-                params["max_id"] + params["count"])
+            params["max_id"] = extr._update_cursor(next_max_id)
 
 
 class InstagramGraphqlAPI():
