@@ -123,6 +123,9 @@ class KemonopartyExtractor(Extractor):
                     g(post) for g in generators):
                 url = file["path"]
 
+                if "\\" in url:
+                    file["path"] = url = url.replace("\\", "/")
+
                 match = find_hash(url)
                 if match:
                     file["hash"] = hash = match.group(1)
@@ -150,7 +153,7 @@ class KemonopartyExtractor(Extractor):
                     file["type"] = "archive"
                     if archives:
                         try:
-                            data = self.api.posts_archives(file["hash"])
+                            data = self.api.file(file["hash"])
                             data.update(file)
                             post_archives.append(data)
                         except Exception as exc:
@@ -314,14 +317,25 @@ class KemonopartyUserExtractor(KemonopartyExtractor):
         KemonopartyExtractor.__init__(self, match)
 
     def posts(self):
+        endpoint = self.config("endpoint")
+        if endpoint == "legacy":
+            endpoint = self.api.creator_posts_legacy
+        elif endpoint == "legacy+":
+            endpoint = self._posts_legacy_plus
+        else:
+            endpoint = self.api.creator_posts
+
         _, _, service, creator_id, query = self.groups
         params = text.parse_query(query)
-        if params.get("tag"):
-            return self.api.creator_tagged_posts(
-                service, creator_id, params.get("tag"), params.get("o"))
-        else:
-            return self.api.creator_posts(
-                service, creator_id, params.get("o"), params.get("q"))
+        return endpoint(service, creator_id,
+                        params.get("o"), params.get("q"), params.get("tag"))
+
+    def _posts_legacy_plus(self, service, creator_id,
+                           offset=0, query=None, tags=None):
+        for post in self.api.creator_posts_legacy(
+                service, creator_id, offset, query, tags):
+            yield self.api.creator_post(
+                service, creator_id, post["id"])["post"]
 
 
 class KemonopartyPostsExtractor(KemonopartyExtractor):
@@ -521,18 +535,20 @@ class KemonoAPI():
         params = {"q": query, "o": offset, "tag": tags}
         return self._pagination(endpoint, params, 50, "posts")
 
-    def posts_archives(self, file_hash):
-        endpoint = "/posts/archives/" + file_hash
-        return self._call(endpoint)["archive"]
+    def file(self, file_hash):
+        endpoint = "/file/" + file_hash
+        return self._call(endpoint)
 
-    def creator_posts(self, service, creator_id, offset=0, query=None):
+    def creator_posts(self, service, creator_id,
+                      offset=0, query=None, tags=None):
         endpoint = "/{}/user/{}".format(service, creator_id)
-        params = {"q": query, "o": offset}
+        params = {"q": query, "tag": tags, "o": offset}
         return self._pagination(endpoint, params, 50)
 
-    def creator_tagged_posts(self, service, creator_id, tags, offset=0):
+    def creator_posts_legacy(self, service, creator_id,
+                             offset=0, query=None, tags=None):
         endpoint = "/{}/user/{}/posts-legacy".format(service, creator_id)
-        params = {"o": offset, "tag": tags}
+        params = {"o": offset, "tag": tags, "q": query}
         return self._pagination(endpoint, params, 50, "results")
 
     def creator_announcements(self, service, creator_id):
