@@ -27,7 +27,7 @@ class PatreonExtractor(Extractor):
     _warning = True
 
     def _init(self):
-        if not self.cookies_check(("session_id",)):
+        if not self.cookies_check(("session_id",), subdomains=True):
             if self._warning:
                 PatreonExtractor._warning = False
                 self.log.warning("no 'session_id' cookie set")
@@ -109,11 +109,7 @@ class PatreonExtractor(Extractor):
 
     def _attachments(self, post):
         for attachment in post.get("attachments") or ():
-            url = self.request(
-                attachment["url"], method="HEAD",
-                allow_redirects=False, fatal=False,
-            ).headers.get("Location")
-
+            url = self.request_location(attachment["url"], fatal=False)
             if url:
                 yield "attachment", url, attachment["name"]
 
@@ -329,10 +325,11 @@ class PatreonCreatorExtractor(PatreonExtractor):
     """Extractor for a creator's works"""
     subcategory = "creator"
     pattern = (r"(?:https?://)?(?:www\.)?patreon\.com"
-               r"/(?!(?:home|join|posts|login|signup)(?:$|[/?#]))"
+               r"/(?!(?:home|create|login|signup|search|posts|messages)"
+               r"(?:$|[/?#]))"
                r"(?:profile/creators|(?:c/)?([^/?#]+)(?:/posts)?)"
                r"/?(?:\?([^#]+))?")
-    example = "https://www.patreon.com/USER"
+    example = "https://www.patreon.com/c/USER"
 
     def posts(self):
         creator, query = self.groups
@@ -370,7 +367,7 @@ class PatreonCreatorExtractor(PatreonExtractor):
             data = None
             data = self._extract_bootstrap(page)
             return data["campaign"]["data"]["id"]
-        except (KeyError, ValueError) as exc:
+        except Exception as exc:
             if data:
                 self.log.debug(data)
             raise exception.StopExtraction(
