@@ -7,7 +7,7 @@
 """Extractors for https://www.weasyl.com/"""
 
 from .common import Extractor, Message
-from .. import text
+from .. import text, util
 
 BASE_PATTERN = r"(?:https://)?(?:www\.)?weasyl.com/"
 
@@ -18,6 +18,7 @@ class WeasylExtractor(Extractor):
     filename_fmt = "{submitid} {title}.{extension}"
     archive_fmt = "{submitid}"
     root = "https://www.weasyl.com"
+    useragent = util.USERAGENT
 
     @staticmethod
     def populate_submission(data):
@@ -30,8 +31,7 @@ class WeasylExtractor(Extractor):
             return True
         return False
 
-    def __init__(self, match):
-        Extractor.__init__(self, match)
+    def _init(self):
         self.session.headers['X-Weasyl-API-Key'] = self.config("api-key")
 
     def request_submission(self, submitid):
@@ -47,6 +47,7 @@ class WeasylExtractor(Extractor):
         return data
 
     def submissions(self, owner_login, folderid=None):
+        metadata = self.config("metadata")
         url = "{}/api/users/{}/gallery".format(self.root, owner_login)
         params = {
             "nextid"  : None,
@@ -56,6 +57,9 @@ class WeasylExtractor(Extractor):
         while True:
             data = self.request(url, params=params).json()
             for submission in data["submissions"]:
+                if metadata:
+                    submission = self.request_submission(
+                        submission["submitid"])
                 if self.populate_submission(submission):
                     submission["folderid"] = folderid
                     # Do any submissions have more than one url? If so
@@ -68,33 +72,8 @@ class WeasylExtractor(Extractor):
 
 class WeasylSubmissionExtractor(WeasylExtractor):
     subcategory = "submission"
-    pattern = BASE_PATTERN + r"(?:~[\w~-]+/submissions|submission)/(\d+)"
-    test = (
-        ("https://www.weasyl.com/~fiz/submissions/2031/a-wesley", {
-            "pattern": "https://cdn.weasyl.com/~fiz/submissions/2031/41ebc1c29"
-                       "40be928532785dfbf35c37622664d2fbb8114c3b063df969562fc5"
-                       "1/fiz-a-wesley.png",
-            "keyword": {
-                "comments"    : int,
-                "date"        : "dt:2012-04-20 00:38:04",
-                "description" : "<p>(flex)</p>\n",
-                "favorites"   : int,
-                "folder_name" : "Wesley Stuff",
-                "folderid"    : 2081,
-                "friends_only": False,
-                "owner"       : "Fiz",
-                "owner_login" : "fiz",
-                "rating"      : "general",
-                "submitid"    : 2031,
-                "subtype"     : "visual",
-                "tags"        : list,
-                "title"       : "A Wesley!",
-                "type"        : "submission",
-                "views"       : int,
-            },
-        }),
-        ("https://www.weasyl.com/submission/2031/a-wesley"),
-    )
+    pattern = BASE_PATTERN + r"(?:~[\w~-]+/submissions|submission|view)/(\d+)"
+    example = "https://www.weasyl.com/~USER/submissions/12345/TITLE"
 
     def __init__(self, match):
         WeasylExtractor.__init__(self, match)
@@ -110,20 +89,13 @@ class WeasylSubmissionExtractor(WeasylExtractor):
 class WeasylSubmissionsExtractor(WeasylExtractor):
     subcategory = "submissions"
     pattern = BASE_PATTERN + r"(?:~|submissions/)([\w~-]+)/?$"
-    test = (
-        ("https://www.weasyl.com/~tanidareal", {
-            "count": ">= 200"
-        }),
-        ("https://www.weasyl.com/submissions/tanidareal"),
-        ("https://www.weasyl.com/~aro~so")
-    )
+    example = "https://www.weasyl.com/submissions/USER"
 
     def __init__(self, match):
         WeasylExtractor.__init__(self, match)
         self.owner_login = match.group(1)
 
     def items(self):
-        yield Message.Version, 1
         yield Message.Directory, {"owner_login": self.owner_login}
         yield from self.submissions(self.owner_login)
 
@@ -132,16 +104,13 @@ class WeasylFolderExtractor(WeasylExtractor):
     subcategory = "folder"
     directory_fmt = ("{category}", "{owner_login}", "{folder_name}")
     pattern = BASE_PATTERN + r"submissions/([\w~-]+)\?folderid=(\d+)"
-    test = ("https://www.weasyl.com/submissions/tanidareal?folderid=7403", {
-        "count": ">= 12"
-    })
+    example = "https://www.weasyl.com/submissions/USER?folderid=12345"
 
     def __init__(self, match):
         WeasylExtractor.__init__(self, match)
         self.owner_login, self.folderid = match.groups()
 
     def items(self):
-        yield Message.Version, 1
         iter = self.submissions(self.owner_login, self.folderid)
         # Folder names are only on single submission api calls
         msg, url, data = next(iter)
@@ -156,14 +125,7 @@ class WeasylJournalExtractor(WeasylExtractor):
     filename_fmt = "{journalid} {title}.{extension}"
     archive_fmt = "{journalid}"
     pattern = BASE_PATTERN + r"journal/(\d+)"
-    test = ("https://www.weasyl.com/journal/17647/bbcode", {
-        "keyword": {
-            "title"  : "BBCode",
-            "date"   : "dt:2013-09-19 23:11:23",
-            "content": "<p><a>javascript:alert(42);</a></p>\n\n"
-                       "<p>No more of that!</p>\n",
-        },
-    })
+    example = "https://www.weasyl.com/journal/12345"
 
     def __init__(self, match):
         WeasylExtractor.__init__(self, match)
@@ -171,7 +133,6 @@ class WeasylJournalExtractor(WeasylExtractor):
 
     def items(self):
         data = self.retrieve_journal(self.journalid)
-        yield Message.Version, 1
         yield Message.Directory, data
         yield Message.Url, data["html"], data
 
@@ -181,16 +142,13 @@ class WeasylJournalsExtractor(WeasylExtractor):
     filename_fmt = "{journalid} {title}.{extension}"
     archive_fmt = "{journalid}"
     pattern = BASE_PATTERN + r"journals/([\w~-]+)"
-    test = ("https://www.weasyl.com/journals/charmander", {
-        "count": ">= 2",
-    })
+    example = "https://www.weasyl.com/journals/USER"
 
     def __init__(self, match):
         WeasylExtractor.__init__(self, match)
         self.owner_login = match.group(1)
 
     def items(self):
-        yield Message.Version, 1
         yield Message.Directory, {"owner_login": self.owner_login}
 
         url = "{}/journals/{}".format(self.root, self.owner_login)
@@ -202,41 +160,45 @@ class WeasylJournalsExtractor(WeasylExtractor):
 
 class WeasylFavoriteExtractor(WeasylExtractor):
     subcategory = "favorite"
-    directory_fmt = ("{category}", "{owner_login}", "Favorites")
-    pattern = BASE_PATTERN + r"favorites\?userid=(\d+)"
-    test = ("https://www.weasyl.com/favorites?userid=184616&feature=submit", {
-        "count": ">= 5",
-    })
-
-    def __init__(self, match):
-        WeasylExtractor.__init__(self, match)
-        self.userid = match.group(1)
+    directory_fmt = ("{category}", "{user}", "Favorites")
+    pattern = BASE_PATTERN + r"favorites(?:\?userid=(\d+)|/([^/?#]+))"
+    example = "https://www.weasyl.com/favorites?userid=12345"
 
     def items(self):
+        userid, username = self.groups
         owner_login = lastid = None
-        url = self.root + "/favorites"
+
+        if username:
+            owner_login = username
+            path = "/favorites/" + username
+        else:
+            path = "/favorites"
         params = {
-            "userid" : self.userid,
+            "userid" : userid,
             "feature": "submit",
         }
 
         while True:
-            page = self.request(url, params=params).text
+            page = self.request(self.root + path, params=params).text
             pos = page.index('id="favorites-content"')
 
             if not owner_login:
-                owner_login = text.extract(page, '<a href="/~', '"')[0]
+                owner_login = text.extr(page, '<a href="/~', '"')
 
             for submitid in text.extract_iter(page, "/submissions/", "/", pos):
                 if submitid == lastid:
                     continue
                 lastid = submitid
+
                 submission = self.request_submission(submitid)
                 if self.populate_submission(submission):
                     submission["user"] = owner_login
                     yield Message.Directory, submission
                     yield Message.Url, submission["url"], submission
 
-            if "&amp;nextid=" not in page:
+            try:
+                pos = page.index('">Next (', pos)
+            except ValueError:
                 return
-            params["nextid"] = submitid
+            path = text.unescape(text.rextract(page, 'href="', '"', pos)[0])
+            params = None
