@@ -30,9 +30,12 @@ class WeiboExtractor(Extractor):
         self._prefix, self.user = match.groups()
 
     def _init(self):
-        self.retweets = self.config("retweets", True)
-        self.videos = self.config("videos", True)
         self.livephoto = self.config("livephoto", True)
+        self.retweets = self.config("retweets", False)
+        self.videos = self.config("videos", True)
+        self.movies = self.config("movies", False)
+        self.gifs = self.config("gifs", True)
+        self.gifs_video = (self.gifs == "video")
 
         cookies = _cookie_cache()
         if cookies is not None:
@@ -41,9 +44,14 @@ class WeiboExtractor(Extractor):
     def request(self, url, **kwargs):
         response = Extractor.request(self, url, **kwargs)
 
-        if response.history and "passport.weibo.com" in response.url:
-            self._sina_visitor_system(response)
-            response = Extractor.request(self, url, **kwargs)
+        if response.history:
+            if "login.sina.com" in response.url:
+                raise exception.StopExtraction(
+                    "HTTP redirect to login page (%s)",
+                    response.url.partition("?")[0])
+            if "passport.weibo.com" in response.url:
+                self._sina_visitor_system(response)
+                response = Extractor.request(self, url, **kwargs)
 
         return response
 
@@ -52,15 +60,25 @@ class WeiboExtractor(Extractor):
 
         for status in self.statuses():
 
-            files = []
-            if self.retweets and "retweeted_status" in status:
+            if "ori_mid" in status and not self.retweets:
+                self.log.debug("Skipping %s (快转 retweet)", status["id"])
+                continue
+
+            if "retweeted_status" in status:
+                if not self.retweets:
+                    self.log.debug("Skipping %s (retweet)", status["id"])
+                    continue
+
+                # videos of the original post are in status
+                # images of the original post are in status["retweeted_status"]
+                files = []
+                self._extract_status(status, files)
+                self._extract_status(status["retweeted_status"], files)
+
                 if original_retweets:
                     status = status["retweeted_status"]
-                    self._extract_status(status, files)
-                else:
-                    self._extract_status(status, files)
-                    self._extract_status(status["retweeted_status"], files)
             else:
+                files = []
                 self._extract_status(status, files)
 
             status["date"] = text.parse_datetime(
@@ -101,16 +119,15 @@ class WeiboExtractor(Extractor):
                 pic = pics[pic_id]
                 pic_type = pic.get("type")
 
-                if pic_type == "gif" and self.videos:
-                    append({"url": pic["video"]})
+                if pic_type == "gif" and self.gifs:
+                    if self.gifs_video:
+                        append({"url": pic["video"]})
+                    else:
+                        append(pic["largest"].copy())
 
                 elif pic_type == "livephoto" and self.livephoto:
                     append(pic["largest"].copy())
-
-                    file = {"url": pic["video"]}
-                    file["filehame"], _, file["extension"] = \
-                        pic["video"].rpartition("%2F")[2].rpartition(".")
-                    append(file)
+                    append({"url": pic["video"]})
 
                 else:
                     append(pic["largest"].copy())
@@ -118,7 +135,10 @@ class WeiboExtractor(Extractor):
         if "page_info" in status:
             info = status["page_info"]
             if "media_info" in info and self.videos:
-                append(self._extract_video(info["media_info"]))
+                if info.get("type") != "5" or self.movies:
+                    append(self._extract_video(info["media_info"]))
+                else:
+                    self.log.debug("%s: Ignoring 'movie' video", status["id"])
 
     def _extract_video(self, info):
         try:
@@ -166,23 +186,34 @@ class WeiboExtractor(Extractor):
 
             data = data["data"]
             statuses = data["list"]
-            if not statuses:
-                return
             yield from statuses
 
-            if "next_cursor" in data:  # videos, newvideo
-                if data["next_cursor"] == -1:
+            # videos, newvideo
+            cursor = data.get("next_cursor")
+            if cursor:
+                if cursor == -1:
                     return
-                params["cursor"] = data["next_cursor"]
-            elif "page" in params:     # home, article
-                params["page"] += 1
-            elif data["since_id"]:     # album
+                params["cursor"] = cursor
+                continue
+
+            # album
+            since_id = data.get("since_id")
+            if since_id:
                 params["sinceid"] = data["since_id"]
-            else:                      # feed, last album page
-                try:
-                    params["since_id"] = statuses[-1]["id"] - 1
-                except KeyError:
+                continue
+
+            # home, article
+            if "page" in params:
+                if not statuses:
                     return
+                params["page"] += 1
+                continue
+
+            # feed, last album page
+            try:
+                params["since_id"] = statuses[-1]["id"] - 1
+            except LookupError:
+                return
 
     def _sina_visitor_system(self, response):
         self.log.info("Sina Visitor System")
@@ -191,7 +222,7 @@ class WeiboExtractor(Extractor):
         headers = {"Referer": response.url}
         data = {
             "cb": "gen_callback",
-            "fp": '{"os":"1","browser":"Gecko91,0,0,0","fonts":"undefined",'
+            "fp": '{"os":"1","browser":"Gecko109,0,0,0","fonts":"undefined",'
                   '"screenInfo":"1920*1080*24","plugins":""}',
         }
 
@@ -203,8 +234,8 @@ class WeiboExtractor(Extractor):
         params = {
             "a"    : "incarnate",
             "t"    : data["tid"],
-            "w"    : "2",
-            "c"    : "{:>03}".format(data["confidence"]),
+            "w"    : "3" if data.get("new_tid") else "2",
+            "c"    : "{:>03}".format(data.get("confidence") or 100),
             "gc"   : "",
             "cb"   : "cross_domain",
             "from" : "weibo",
@@ -220,8 +251,10 @@ class WeiboUserExtractor(WeiboExtractor):
     pattern = USER_PATTERN + r"(?:$|#)"
     example = "https://weibo.com/USER"
 
-    def initialize(self):
-        pass
+    # do NOT override 'initialize()'
+    # it is needed for 'self._user_id()'
+    # def initialize(self):
+    #     pass
 
     def items(self):
         base = "{}/u/{}?tabtype=".format(self.root, self._user_id())
