@@ -325,6 +325,23 @@ def _parse_slice(format_spec, default):
     return apply_slice
 
 
+def _parse_arithmetic(format_spec, default):
+    op, _, format_spec = format_spec.partition(_SEPARATOR)
+    fmt = _build_format_func(format_spec, default)
+
+    value = int(op[2:])
+    op = op[1]
+
+    if op == "+":
+        return lambda obj: fmt(obj + value)
+    if op == "-":
+        return lambda obj: fmt(obj - value)
+    if op == "*":
+        return lambda obj: fmt(obj * value)
+
+    return fmt
+
+
 def _parse_conversion(format_spec, default):
     conversions, _, format_spec = format_spec.partition(_SEPARATOR)
     convs = [_CONVERSIONS[c] for c in conversions[1:]]
@@ -459,6 +476,7 @@ _GLOBALS = {
     "_env": lambda: os.environ,
     "_lit": lambda: _literal,
     "_now": datetime.datetime.now,
+    "_nul": lambda: util.NONE,
 }
 _CONVERSIONS = {
     "l": str.lower,
@@ -467,6 +485,7 @@ _CONVERSIONS = {
     "C": string.capwords,
     "j": util.json_dumps,
     "t": str.strip,
+    "L": len,
     "T": util.datetime_to_timestamp_string,
     "d": text.parse_timestamp,
     "U": text.unescape,
@@ -476,10 +495,13 @@ _CONVERSIONS = {
     "s": str,
     "r": repr,
     "a": ascii,
+    "i": int,
+    "f": float,
 }
 _FORMAT_SPECIFIERS = {
     "?": _parse_optional,
     "[": _parse_slice,
+    "A": _parse_arithmetic,
     "C": _parse_conversion,
     "D": _parse_datetime,
     "J": _parse_join,
