@@ -10,6 +10,7 @@
 
 import argparse
 import logging
+import os.path
 import sys
 from . import job, util, version
 
@@ -152,6 +153,53 @@ class UgoiraAction(argparse.Action):
         namespace.postprocessors.append(pp)
 
 
+class PrintAction(argparse.Action):
+    def __call__(self, parser, namespace, value, option_string=None):
+        if self.const:
+            filename = self.const
+            base = None
+            mode = "w"
+        else:
+            value, path = value
+            base, filename = os.path.split(path)
+            mode = "a"
+
+        event, sep, format_string = value.partition(":")
+        if not sep:
+            format_string = event
+            event = ("prepare",)
+        else:
+            event = event.strip().lower()
+            if event not in {"init", "file", "after", "skip", "error",
+                             "prepare", "prepare-after", "post", "post-after",
+                             "finalize", "finalize-success", "finalize-error"}:
+                format_string = value
+                event = ("prepare",)
+
+        if not format_string:
+            return
+
+        if format_string.startswith("\\f"):
+            format_string = "\f" + format_string[2:]
+
+        if format_string[0] == "\f":
+            if format_string[1] == "F" and format_string[-1] != "\n":
+                format_string += "\n"
+        elif "{" not in format_string and " " not in format_string:
+            format_string = "{" + format_string + "}\n"
+        elif format_string[-1] != "\n":
+            format_string += "\n"
+
+        namespace.postprocessors.append({
+            "name"          : "metadata",
+            "event"         : event,
+            "filename"      : filename,
+            "base-directory": base or ".",
+            "content-format": format_string,
+            "open"          : mode,
+        })
+
+
 class Formatter(argparse.HelpFormatter):
     """Custom HelpFormatter class to customize help output"""
     def __init__(self, prog):
@@ -279,7 +327,7 @@ def build_parser():
     input.add_argument(
         "--no-input",
         dest="input", nargs=0, action=ConfigConstAction, const=False,
-        help=("Do not prompt for passwords/tokens"),
+        help="Do not prompt for passwords/tokens",
     )
 
     output = parser.add_argument_group("Output Options")
@@ -342,6 +390,19 @@ def build_parser():
         dest="errorfile", metavar="FILE", action=ConfigAction,
         help="Add input URLs which returned an error to FILE",
     )
+    output.add_argument(
+        "-N", "--print",
+        dest="postprocessors", metavar="[EVENT:]FORMAT",
+        action=PrintAction, const="-", default=[],
+        help=("Write FORMAT during EVENT (default 'prepare') to standard "
+              "output. Examples: 'id' or 'post:{md5[:8]}'"),
+    )
+    output.add_argument(
+        "--print-to-file",
+        dest="postprocessors", metavar="[EVENT:]FORMAT FILE",
+        action=PrintAction, nargs=2,
+        help="Append FORMAT during EVENT to FILE",
+    )
     output.add_argument(
         "--list-modules",
         dest="list_modules", action="store_true",
@@ -349,7 +410,7 @@ def build_parser():
     )
     output.add_argument(
         "--list-extractors",
-        dest="list_extractors", metavar="CATEGORIES", nargs="*",
+        dest="list_extractors", metavar="[CATEGORIES]", nargs="*",
         help=("Print a list of extractor classes "
               "with description, (sub)category and example URL"),
     )
@@ -373,12 +434,12 @@ def build_parser():
     output.add_argument(
         "--print-traffic",
         dest="print_traffic", action="store_true",
-        help=("Display sent and read HTTP traffic"),
+        help="Display sent and read HTTP traffic",
     )
     output.add_argument(
         "--no-colors",
         dest="colors", action="store_false",
-        help=("Do not emit ANSI color codes in output"),
+        help="Do not emit ANSI color codes in output",
     )
 
     networking = parser.add_argument_group("Networking Options")
@@ -403,6 +464,17 @@ def build_parser():
         dest="source-address", metavar="IP", action=ConfigAction,
         help="Client-side IP address to bind to",
     )
+    networking.add_argument(
+        "-4", "--force-ipv4",
+        dest="source-address", nargs=0, action=ConfigConstAction,
+        const="0.0.0.0",
+        help="Make all connections via IPv4",
+    )
+    networking.add_argument(
+        "-6", "--force-ipv6",
+        dest="source-address", nargs=0, action=ConfigConstAction, const="::",
+        help="Make all connections via IPv6",
+    )
     networking.add_argument(
         "--no-check-certificate",
         dest="verify", nargs=0, action=ConfigConstAction, const=False,
@@ -616,7 +688,7 @@ def build_parser():
     postprocessor = parser.add_argument_group("Post-processing Options")
     postprocessor.add_argument(
         "-P", "--postprocessor",
-        dest="postprocessors", metavar="NAME", action="append", default=[],
+        dest="postprocessors", metavar="NAME", action="append",
         help="Activate the specified post processor",
     )
     postprocessor.add_argument(
@@ -735,4 +807,10 @@ def build_parser():
               "&& convert * ../doc.pdf\""),
     )
 
+    try:
+        # restore normal behavior when adding '-4' or '-6' as arguments
+        parser._has_negative_number_optionals.clear()
+    except Exception:
+        pass
+
     return parser
