@@ -160,10 +160,10 @@ def extract_all(txt, rules, pos=0, values=None):
 
 def extract_iter(txt, begin, end, pos=0):
     """Yield values that would be returned by repeated calls of extract()"""
-    index = txt.index
-    lbeg = len(begin)
-    lend = len(end)
     try:
+        index = txt.index
+        lbeg = len(begin)
+        lend = len(end)
         while True:
             first = index(begin, pos) + lbeg
             last = index(end, first)
@@ -238,12 +238,48 @@ def parse_float(value, default=0.0):
 
 
 def parse_query(qs):
-    """Parse a query string into key-value pairs"""
+    """Parse a query string into name-value pairs
+
+    Ignore values whose name has been seen before
+    """
+    if not qs:
+        return {}
+
     result = {}
     try:
-        for key, value in urllib.parse.parse_qsl(qs):
-            if key not in result:
-                result[key] = value
+        for name_value in qs.split("&"):
+            name, eq, value = name_value.partition("=")
+            if eq:
+                name = unquote(name.replace("+", " "))
+                if name not in result:
+                    result[name] = unquote(value.replace("+", " "))
+    except Exception:
+        pass
+    return result
+
+
+def parse_query_list(qs, as_list=()):
+    """Parse a query string into name-value pairs
+
+    Combine values of names in 'as_list' into lists
+    """
+    if not qs:
+        return {}
+
+    result = {}
+    try:
+        for name_value in qs.split("&"):
+            name, eq, value = name_value.partition("=")
+            if eq:
+                name = unquote(name.replace("+", " "))
+                value = unquote(value.replace("+", " "))
+                if name in as_list:
+                    if name in result:
+                        result[name].append(value)
+                    else:
+                        result[name] = [value]
+                elif name not in result:
+                    result[name] = unquote(value.replace("+", " "))
     except Exception:
         pass
     return result
