@@ -173,6 +173,24 @@ class ClassifyTest(BasePostprocessorTest):
         self.assertEqual(self.pathfmt.realpath, path + "/file.foo")
 
 
+class DirectoryTest(BasePostprocessorTest):
+
+    def test_default(self):
+        self._create()
+
+        path = os.path.join(self.dir.name, "test")
+        self.assertEqual(self.pathfmt.realdirectory, path + "/")
+        self.assertEqual(self.pathfmt.realpath, path + "/file.ext")
+
+        self.pathfmt.kwdict["category"] = "custom"
+        self._trigger()
+
+        path = os.path.join(self.dir.name, "custom")
+        self.assertEqual(self.pathfmt.realdirectory, path + "/")
+        self.pathfmt.build_path()
+        self.assertEqual(self.pathfmt.realpath, path + "/file.ext")
+
+
 class ExecTest(BasePostprocessorTest):
 
     def test_command_string(self):
@@ -511,6 +529,17 @@ class MetadataTest(BasePostprocessorTest):
         path = self.pathfmt.realdirectory + "../json/12500/file.ext.json"
         m.assert_called_once_with(path, "w", encoding="utf-8")
 
+    def test_metadata_directory_empty(self):
+        self._create(
+            {"directory": []},
+        )
+
+        with patch("builtins.open", mock_open()) as m:
+            self._trigger()
+
+        path = self.pathfmt.realdirectory + "./file.ext.json"
+        m.assert_called_once_with(path, "w", encoding="utf-8")
+
     def test_metadata_basedirectory(self):
         self._create({"base-directory": True})
 
@@ -544,6 +573,16 @@ class MetadataTest(BasePostprocessorTest):
         path = self.pathfmt.realdirectory + "test_file__meta_.data"
         m.assert_called_once_with(path, "w", encoding="utf-8")
 
+    def test_metadata_meta_path(self):
+        self._create({
+            "metadata-path": "_meta_path",
+        })
+
+        self._trigger()
+
+        self.assertEqual(self.pathfmt.kwdict["_meta_path"],
+                         self.pathfmt.realpath + ".json")
+
     def test_metadata_stdout(self):
         self._create({"filename": "-", "indent": None, "sort": True})
 
