@@ -92,6 +92,15 @@ class TestExtractorResults(unittest.TestCase):
             self.assertGreaterEqual(value, range.start, msg=msg)
 
     def _run_test(self, result):
+        if result.get("#fail"):
+            del result["#fail"]
+            try:
+                self._run_test(result)
+            except AssertionError:
+                return
+            else:
+                self.fail("Test did not fail")
+
         base, cat, sub = result_categories(result)
         result.pop("#comment", None)
         result.pop("#category", None)
@@ -106,6 +115,10 @@ class TestExtractorResults(unittest.TestCase):
         if len(result) <= 2:
             return  # only matching
 
+        skip = result.pop("#skip", False)
+        if skip:
+            return self._skipped.append((result["#url"], skip))
+
         if auth is None:
             auth = (cat in AUTH_REQUIRED)
         elif not auth:
@@ -226,7 +239,11 @@ class TestExtractorResults(unittest.TestCase):
                 key = key[1:]
                 if key not in kwdict:
                     continue
+
             path = "{}.{}".format(parent, key) if parent else key
+            if key.startswith("!"):
+                self.assertNotIn(key[1:], kwdict, msg=path)
+                continue
             self.assertIn(key, kwdict, msg=path)
             value = kwdict[key]
 
@@ -237,7 +254,10 @@ class TestExtractorResults(unittest.TestCase):
             elif isinstance(test, range):
                 self.assertRange(value, test, msg=path)
             elif isinstance(test, set):
-                self.assertTrue(value in test or type(value) in test, msg=path)
+                try:
+                    self.assertIn(value, test, msg=path)
+                except AssertionError:
+                    self.assertIn(type(value), test, msg=path)
             elif isinstance(test, list):
                 subtest = False
                 for idx, item in enumerate(test):
@@ -256,8 +276,11 @@ class TestExtractorResults(unittest.TestCase):
                 elif test.startswith("type:"):
                     self.assertEqual(test[5:], type(value).__name__, msg=path)
                 elif test.startswith("len:"):
-                    self.assertIsInstance(value, (list, tuple), msg=path)
-                    self.assertEqual(int(test[4:]), len(value), msg=path)
+                    cls, _, length = test[4:].rpartition(":")
+                    if cls:
+                        self.assertEqual(
+                            cls, type(value).__name__, msg=path + "/type")
+                    self.assertEqual(int(length), len(value), msg=path)
                 else:
                     self.assertEqual(test, value, msg=path)
             else:
