@@ -22,7 +22,7 @@ class Test_CommandlineArguments(unittest.TestCase):
     def setUpClass(cls):
         try:
             cls.module = __import__(cls.module_name)
-        except ImportError:
+        except (ImportError, SyntaxError):
             raise unittest.SkipTest("cannot import module '{}'".format(
                 cls.module_name))
         cls.default = ytdl.parse_command_line(cls.module, [])
@@ -294,6 +294,20 @@ class Test_CommandlineArguments_YtDlp(Test_CommandlineArguments):
         self._(["--geo-bypass-ip-block", "198.51.100.14/24"],
                "geo_bypass", "198.51.100.14/24")
 
+    def test_cookiesfrombrowser(self):
+        self._(["--cookies-from-browser", "firefox"],
+               "cookiesfrombrowser", ("firefox", None, None, None))
+        self._(["--cookies-from-browser", "firefox:profile"],
+               "cookiesfrombrowser", ("firefox", "profile", None, None))
+        self._(["--cookies-from-browser", "firefox+keyring"],
+               "cookiesfrombrowser", ("firefox", None, "KEYRING", None))
+        self._(["--cookies-from-browser", "firefox::container"],
+               "cookiesfrombrowser", ("firefox", None, None, "container"))
+        self._(["--cookies-from-browser",
+                "firefox+keyring:profile::container"],
+               "cookiesfrombrowser",
+               ("firefox", "profile", "KEYRING", "container"))
+
 
 if __name__ == "__main__":
     unittest.main(warnings="ignore")
